==> llm-docs-0.1.2.tgz: package/docs/architecture/SHARED_COMPONENTS.md <==
---
id: architecture-SHARED_COMPONENTS
---
# Shared Components Reference

This document provides detailed information about the shared utilities available in the `@bluefly/bfcli/shared` package. These components provide standardized implementations of common functionality across Bluefly projects.

## Error Handling

### Overview

The error handling system provides a standardized approach to creating, managing, and responding to errors across all Bluefly projects. It includes predefined error types, error creation utilities, and middleware for handling errors in HTTP contexts.

### Key Components

#### `BlueflyError` Interface

```typescript
export interface BlueflyError extends Error {
  code: string;
  statusCode?: number;
  details?: Record<string, any>;
}
```

#### Error Creation

```typescript
// Create a standardized error
import { createError, ERROR_CODES } from '@bluefly/bfcli/shared';

const error = createError(
  'Failed to connect to service',
  ERROR_CODES.SERVICE_UNAVAILABLE,
  503,
  { service: 'api', attempt: 3 }
);
```

#### Standard Error Types

```typescript
import {
  ValidationError,
  NotFoundError,
  UnauthorizedError,
  ForbiddenError,
  ConflictError,
  RateLimitError,
  ServiceUnavailableError,
  InternalServerError
} from '@bluefly/bfcli/shared';

throw new ValidationError('Invalid input data');
throw new NotFoundError('User not found');
throw new UnauthorizedError('Authentication required');
```

#### Error Handler Middleware

```typescript
import { errorHandlerMiddleware } from '@bluefly/bfcli/shared';
import express from 'express';

const app = express();

// Add error handler middleware
app.use(errorHandlerMiddleware({
  logger: customLogger,
  includeStackTrace: process.env.NODE_ENV !== 'production'
}));
```

#### Error Utilities

```typescript
import { errorUtils } from '@bluefly/bfcli/shared';

// Determine if error is retryable
if (errorUtils.isRetryableError(error)) {
  // Retry operation
}

// Translate error from third-party library
const standardError = errorUtils.translateError(
  originalError,
  'SERVICE_NAME'
);

// Format error for response
const errorResponse = errorUtils.formatErrorResponse(error);
```

### Error Codes

| Code | Description | HTTP Status |
|------|-------------|-------------|
| `VALIDATION_ERROR` | Input validation failed | 400 |
| `UNAUTHORIZED` | Authentication required | 401 |
| `FORBIDDEN` | Insufficient permissions | 403 |
| `NOT_FOUND` | Resource not found | 404 |
| `CONFLICT` | Resource conflict | 409 |
| `API_RATE_LIMIT` | Rate limit exceeded | 429 |
| `INVALID_CONFIGURATION` | Configuration error | 500 |
| `SERVICE_UNAVAILABLE` | Service temporarily unavailable | 503 |
| `UNKNOWN_ERROR` | Unclassified error | 500 |

## Logger

### Overview

The logging system provides consistent, structured logging across all Bluefly projects with support for different log levels, context tracking, and output formatting.

### Key Components

#### `BlueflyLogger` Class

```typescript
import { BlueflyLogger, LogLevel } from '@bluefly/bfcli/shared';

const logger = new BlueflyLogger({
  prefix: '[my-service]',
  level: 'info',
  timestamp: true,
  colorize: true
});

logger.info('Operation completed');
logger.error('Operation failed', { error: new Error('Connection error') });
```

#### Log Levels

- `debug`: Detailed debugging information
- `info`: Normal operation information
- `warn`: Warning conditions
- `error`: Error conditions

#### Contextual Logging

```typescript
// Log with context
logger.info('User created', {
  userId: 'user123',
  email: 'user@example.com',
  correlationId: 'abc-123'
});

// Log error with details
logger.error('Database operation failed', {
  operation: 'insertUser',
  error: dbError,
  correlationId: 'abc-123'
});
```

#### Default Logger

```typescript
import { logger } from '@bluefly/bfcli/shared';

// Use the default shared logger
logger.info('Using default shared logger');
```

### Configuration

```typescript
// Environment variable configuration
process.env.LOG_LEVEL = 'debug';
process.env.LOG_FORMAT = 'json';
process.env.LOG_COLORIZE = 'false';
```

## HTTP Client

### Overview

The HTTP client provides a resilient, feature-rich client for making HTTP requests with built-in retry logic, circuit breaker pattern, and standardized error handling.

### Key Components

#### `HttpClient` Class

```typescript
import { HttpClient } from '@bluefly/bfcli/shared';

const httpClient = new HttpClient({
  baseURL: 'https://api.example.com',
  timeout: 5000,
  retries: 3,
  retryDelay: 300,
  circuitBreaker: {
    enabled: true,
    failureThreshold: 5,
    resetTimeout: 30000
  },
  headers: {
    'X-API-Key': 'api-key-here'
  }
});
```

#### Making Requests

```typescript
// GET request
const response = await httpClient.get('/users', {
  params: { active: true },
  context: { correlationId: 'abc-123' }
});

// POST request
const createResponse = await httpClient.post('/users', {
  name: 'John Doe',
  email: 'john@example.com'
}, {
  headers: { 'X-Custom-Header': 'value' },
  context: { correlationId: 'abc-123' }
});
```

#### Retry Configuration

```typescript
// Custom retry configuration
const response = await httpClient.get('/unreliable-endpoint', {
  retry: {
    retries: 5,
    retryDelay: 1000,
    retryStatusCodes: [408, 429, 500, 502, 503, 504],
    shouldRetry: (error, attempt) => {
      // Custom retry logic
      return attempt < 3 && error.statusCode === 500;
    }
  }
});
```

#### Circuit Breaker

The circuit breaker pattern prevents cascading failures by temporarily blocking requests to failing services.

```typescript
const httpClient = new HttpClient({
  baseURL: 'https://api.example.com',
  circuitBreaker: {
    enabled: true,
    failureThreshold: 5,      // Number of failures before opening circuit
    resetTimeout: 30000,      // Time in ms before attempting to close circuit
    halfOpenSuccessThreshold: 2  // Successes needed in half-open state to close
  }
});
```

#### Interceptors

```typescript
// Request interceptor
httpClient.addRequestInterceptor((config) => {
  config.headers['X-Request-Time'] = Date.now();
  return config;
});

// Response interceptor
httpClient.addResponseInterceptor((response) => {
  console.log(`Response time: ${Date.now() - response.config.headers['X-Request-Time']}ms`);
  return response;
});
```

### Error Handling

All HTTP errors are automatically converted to standardized BlueflyError instances:

```typescript
try {
  const response = await httpClient.get('/nonexistent');
} catch (error) {
  console.log(error.code);      // 'NOT_FOUND'
  console.log(error.statusCode); // 404
  console.log(error.details);   // Additional context about the error
}
```

## Observability

### Overview

The observability system provides tools for monitoring application performance, tracking requests, and collecting metrics.

### Key Components

#### Metrics

```typescript
import { metrics } from '@bluefly/bfcli/shared';

// Increment counter
metrics.counter('api_requests_total', 1, {
  method: 'GET',
  path: '/users',
  status: 200
});

// Record timing
metrics.timing('request_duration_ms', 235, {
  service: 'database',
  operation: 'query'
});

// Record gauge
metrics.gauge('active_connections', 42, {
  pool: 'main'
});
```

#### Tracing

```typescript
import { tracer } from '@bluefly/bfcli/shared';

// Create a span
const span = tracer.startSpan('process_request', {
  parent: parentContext,
  attributes: {
    'http.method': 'GET',
    'http.url': '/users'
  }
});

// Add events to span
span.addEvent('cache_check', { cacheHit: false });

// Set span status
span.setStatus({ code: tracer.SpanStatusCode.OK });

// End the span
span.end();
```

#### Health Checks

```typescript
import { healthCheck } from '@bluefly/bfcli/shared';

// Register health checks
healthCheck.register('database', async () => {
  return dbClient.isConnected();
});

healthCheck.register('api', async () => {
  const response = await fetch('https://api.example.com/health');
  return response.ok;
});

// Express middleware
app.use('/health', healthCheck.middleware());
```

## General Utilities

### String Utilities

```typescript
import { stringUtils } from '@bluefly/bfcli/shared';

// Camel case conversion
stringUtils.camelCase('hello-world');  // 'helloWorld'

// Generate slug
stringUtils.slugify('Hello World!');  // 'hello-world'

// Truncate string
stringUtils.truncate('A very long string', 10);  // 'A very lon...'
```

### Object Utilities

```typescript
import { objectUtils } from '@bluefly/bfcli/shared';

// Deep merge
objectUtils.deepMerge({ a: 1, b: { c: 2 } }, { b: { d: 3 } });
// { a: 1, b: { c: 2, d: 3 } }

// Pick properties
objectUtils.pick({ a: 1, b: 2, c: 3 }, ['a', 'c']);
// { a: 1, c: 3 }

// Omit properties
objectUtils.omit({ a: 1, b: 2, c: 3 }, ['b']);
// { a: 1, c: 3 }
```

### ID Utilities

```typescript
import { idUtils } from '@bluefly/bfcli/shared';

// Generate UUID
const uuid = idUtils.generateUuid();

// Generate short ID
const shortId = idUtils.generateShortId();

// Generate correlation ID
const correlationId = idUtils.generateCorrelationId();
```

### Date Utilities

```typescript
import { dateUtils } from '@bluefly/bfcli/shared';

// Format date
dateUtils.formatDate(new Date(), 'yyyy-MM-dd');  // '2023-05-15'

// Add time
const tomorrow = dateUtils.addDays(new Date(), 1);
const nextHour = dateUtils.addHours(new Date(), 1);

// Calculate difference
const diffDays = dateUtils.diffInDays(startDate, endDate);
```

### Validation Utilities

```typescript
import { validationUtils } from '@bluefly/bfcli/shared';

// String validation
const isValidEmail = validationUtils.string.isEmail('user@example.com');
const isValidUrl = validationUtils.string.isUrl('https://example.com');
const isValidUuid = validationUtils.string.isUuid('123e4567-e89b-12d3-a456-426614174000');

// Number validation
const isPositive = validationUtils.number.isPositive(42);
const inRange = validationUtils.number.inRange(25, 0, 100);

// Object schema validation
const result = validationUtils.object.matchesSchema(config, {
  baseUrl: { type: 'string', required: true },
  timeout: { type: 'number', min: 0 },
  retries: { type: 'number', min: 0, max: 10 }
});

// Array validation
const hasMinLength = validationUtils.array.minLength([1, 2, 3], 2);
const noDuplicates = validationUtils.array.noDuplicates([1, 2, 3, 4]);

// Validate or throw
try {
  validationUtils.assertValid(
    password,
    [
      (pwd) => validationUtils.string.hasMinLength(pwd, 8),
      (pwd) => /[A-Z]/.test(pwd) // Has uppercase letter
    ],
    'Invalid password'
  );
} catch (error) {
  // Handle validation error
}
```

## Integration

### Package Installation

```bash
npm install @bluefly/bfcli
```

### ES Module Import

```typescript
// Import with .js extension for ESM compatibility
import { logger } from '@bluefly/bfcli/shared/logger/index.js';
import { createError } from '@bluefly/bfcli/shared/errors/index.js';
import { HttpClient } from '@bluefly/bfcli/shared/http/index.js';
```

### CommonJS Import

```javascript
// CommonJS fallback import
const { logger } = require('@bluefly/bfcli/shared/logger/index.js');
const { createError } = require('@bluefly/bfcli/shared/errors/index.js');
const { HttpClient } = require('@bluefly/bfcli/shared/http/index.js');
```

==> llm-docs-0.1.5.tgz: package/docs/architecture/SHARED_COMPONENTS.md <==
---
id: architecture-SHARED_COMPONENTS
---
# Shared Components Reference

This document provides detailed information about the shared utilities available in the `@bluefly/llmcli/shared` package. These components provide standardized implementations of common functionality across LLM Platform projects.

## Error Handling

### Overview

The error handling system provides a standardized approach to creating, managing, and responding to errors across all LLM Platform projects. It includes predefined error types, error creation utilities, and middleware for handling errors in HTTP contexts.

### Key Components

#### `LLM PlatformError` Interface

```typescript
export interface LLM PlatformError extends Error {
  code: string;
  statusCode?: number;
  details?: Record<string, any>;
}
```

#### Error Creation

```typescript
// Create a standardized error
import { createError, ERROR_CODES } from '@bluefly/llmcli/shared';

const error = createError(
  'Failed to connect to service',
  ERROR_CODES.SERVICE_UNAVAILABLE,
  503,
  { service: 'api', attempt: 3 }
);
```

#### Standard Error Types

```typescript
import {
  ValidationError,
  NotFoundError,
  UnauthorizedError,
  ForbiddenError,
  ConflictError,
  RateLimitError,
  ServiceUnavailableError,
  InternalServerError
} from '@bluefly/llmcli/shared';

throw new ValidationError('Invalid input data');
throw new NotFoundError('User not found');
throw new UnauthorizedError('Authentication required');
```

#### Error Handler Middleware

```typescript
import { errorHandlerMiddleware } from '@bluefly/llmcli/shared';
import express from 'express';

const app = express();

// Add error handler middleware
app.use(errorHandlerMiddleware({
  logger: customLogger,
  includeStackTrace: process.env.NODE_ENV !== 'production'
}));
```

#### Error Utilities

```typescript
import { errorUtils } from '@bluefly/llmcli/shared';

// Determine if error is retryable
if (errorUtils.isRetryableError(error)) {
  // Retry operation
}

// Translate error from third-party library
const standardError = errorUtils.translateError(
  originalError,
  'SERVICE_NAME'
);

// Format error for response
const errorResponse = errorUtils.formatErrorResponse(error);
```

### Error Codes

| Code | Description | HTTP Status |
|------|-------------|-------------|
| `VALIDATION_ERROR` | Input validation failed | 400 |
| `UNAUTHORIZED` | Authentication required | 401 |
| `FORBIDDEN` | Insufficient permissions | 403 |
| `NOT_FOUND` | Resource not found | 404 |
| `CONFLICT` | Resource conflict | 409 |
| `API_RATE_LIMIT` | Rate limit exceeded | 429 |
| `INVALID_CONFIGURATION` | Configuration error | 500 |
| `SERVICE_UNAVAILABLE` | Service temporarily unavailable | 503 |
| `UNKNOWN_ERROR` | Unclassified error | 500 |

## Logger

### Overview

The logging system provides consistent, structured logging across all LLM Platform projects with support for different log levels, context tracking, and output formatting.

### Key Components

#### `LLM PlatformLogger` Class

```typescript
import { LLM PlatformLogger, LogLevel } from '@bluefly/llmcli/shared';

const logger = new LLM PlatformLogger({
  prefix: '[my-service]',
  level: 'info',
  timestamp: true,
  colorize: true
});

logger.info('Operation completed');
logger.error('Operation failed', { error: new Error('Connection error') });
```

#### Log Levels

- `debug`: Detailed debugging information
- `info`: Normal operation information
- `warn`: Warning conditions
- `error`: Error conditions

#### Contextual Logging

```typescript
// Log with context
logger.info('User created', {
  userId: 'user123',
  email: 'user@example.com',
  correlationId: 'abc-123'
});

// Log error with details
logger.error('Database operation failed', {
  operation: 'insertUser',
  error: dbError,
  correlationId: 'abc-123'
});
```

#### Default Logger

```typescript
import { logger } from '@bluefly/llmcli/shared';

// Use the default shared logger
logger.info('Using default shared logger');
```

### Configuration

```typescript
// Environment variable configuration
process.env.LOG_LEVEL = 'debug';
process.env.LOG_FORMAT = 'json';
process.env.LOG_COLORIZE = 'false';
```

## HTTP Client

### Overview

The HTTP client provides a resilient, feature-rich client for making HTTP requests with built-in retry logic, circuit breaker pattern, and standardized error handling.

### Key Components

#### `HttpClient` Class

```typescript
import { HttpClient } from '@bluefly/llmcli/shared';

const httpClient = new HttpClient({
  baseURL: 'https://api.example.com',
  timeout: 5000,
  retries: 3,
  retryDelay: 300,
  circuitBreaker: {
    enabled: true,
    failureThreshold: 5,
    resetTimeout: 30000
  },
  headers: {
    'X-API-Key': 'api-key-here'
  }
});
```

#### Making Requests

```typescript
// GET request
const response = await httpClient.get('/users', {
  params: { active: true },
  context: { correlationId: 'abc-123' }
});

// POST request
const createResponse = await httpClient.post('/users', {
  name: 'John Doe',
  email: 'john@example.com'
}, {
  headers: { 'X-Custom-Header': 'value' },
  context: { correlationId: 'abc-123' }
});
```

#### Retry Configuration

```typescript
// Custom retry configuration
const response = await httpClient.get('/unreliable-endpoint', {
  retry: {
    retries: 5,
    retryDelay: 1000,
    retryStatusCodes: [408, 429, 500, 502, 503, 504],
    shouldRetry: (error, attempt) => {
      // Custom retry logic
      return attempt < 3 && error.statusCode === 500;
    }
  }
});
```

#### Circuit Breaker

The circuit breaker pattern prevents cascading failures by temporarily blocking requests to failing services.

```typescript
const httpClient = new HttpClient({
  baseURL: 'https://api.example.com',
  circuitBreaker: {
    enabled: true,
    failureThreshold: 5,      // Number of failures before opening circuit
    resetTimeout: 30000,      // Time in ms before attempting to close circuit
    halfOpenSuccessThreshold: 2  // Successes needed in half-open state to close
  }
});
```

#### Interceptors

```typescript
// Request interceptor
httpClient.addRequestInterceptor((config) => {
  config.headers['X-Request-Time'] = Date.now();
  return config;
});

// Response interceptor
httpClient.addResponseInterceptor((response) => {
  console.log(`Response time: ${Date.now() - response.config.headers['X-Request-Time']}ms`);
  return response;
});
```

### Error Handling

All HTTP errors are automatically converted to standardized LLM PlatformError instances:

```typescript
try {
  const response = await httpClient.get('/nonexistent');
} catch (error) {
  console.log(error.code);      // 'NOT_FOUND'
  console.log(error.statusCode); // 404
  console.log(error.details);   // Additional context about the error
}
```

## Observability

### Overview

The observability system provides tools for monitoring application performance, tracking requests, and collecting metrics.

### Key Components

#### Metrics

```typescript
import { metrics } from '@bluefly/llmcli/shared';

// Increment counter
metrics.counter('api_requests_total', 1, {
  method: 'GET',
  path: '/users',
  status: 200
});

// Record timing
metrics.timing('request_duration_ms', 235, {
  service: 'database',
  operation: 'query'
});

// Record gauge
metrics.gauge('active_connections', 42, {
  pool: 'main'
});
```

#### Tracing

```typescript
import { tracer } from '@bluefly/llmcli/shared';

// Create a span
const span = tracer.startSpan('process_request', {
  parent: parentContext,
  attributes: {
    'http.method': 'GET',
    'http.url': '/users'
  }
});

// Add events to span
span.addEvent('cache_check', { cacheHit: false });

// Set span status
span.setStatus({ code: tracer.SpanStatusCode.OK });

// End the span
span.end();
```

#### Health Checks

```typescript
import { healthCheck } from '@bluefly/llmcli/shared';

// Register health checks
healthCheck.register('database', async () => {
  return dbClient.isConnected();
});

healthCheck.register('api', async () => {
  const response = await fetch('https://api.example.com/health');
  return response.ok;
});

// Express middleware
app.use('/health', healthCheck.middleware());
```

## General Utilities

### String Utilities

```typescript
import { stringUtils } from '@bluefly/llmcli/shared';

// Camel case conversion
stringUtils.camelCase('hello-world');  // 'helloWorld'

// Generate slug
stringUtils.slugify('Hello World!');  // 'hello-world'

// Truncate string
stringUtils.truncate('A very long string', 10);  // 'A very lon...'
```

### Object Utilities

```typescript
import { objectUtils } from '@bluefly/llmcli/shared';

// Deep merge
objectUtils.deepMerge({ a: 1, b: { c: 2 } }, { b: { d: 3 } });
// { a: 1, b: { c: 2, d: 3 } }

// Pick properties
objectUtils.pick({ a: 1, b: 2, c: 3 }, ['a', 'c']);
// { a: 1, c: 3 }

// Omit properties
objectUtils.omit({ a: 1, b: 2, c: 3 }, ['b']);
// { a: 1, c: 3 }
```

### ID Utilities

```typescript
import { idUtils } from '@bluefly/llmcli/shared';

// Generate UUID
const uuid = idUtils.generateUuid();

// Generate short ID
const shortId = idUtils.generateShortId();

// Generate correlation ID
const correlationId = idUtils.generateCorrelationId();
```

### Date Utilities

```typescript
import { dateUtils } from '@bluefly/llmcli/shared';

// Format date
dateUtils.formatDate(new Date(), 'yyyy-MM-dd');  // '2023-05-15'

// Add time
const tomorrow = dateUtils.addDays(new Date(), 1);
const nextHour = dateUtils.addHours(new Date(), 1);

// Calculate difference
const diffDays = dateUtils.diffInDays(startDate, endDate);
```

### Validation Utilities

```typescript
import { validationUtils } from '@bluefly/llmcli/shared';

// String validation
const isValidEmail = validationUtils.string.isEmail('user@example.com');
const isValidUrl = validationUtils.string.isUrl('https://example.com');
const isValidUuid = validationUtils.string.isUuid('123e4567-e89b-12d3-a456-426614174000');

// Number validation
const isPositive = validationUtils.number.isPositive(42);
const inRange = validationUtils.number.inRange(25, 0, 100);

// Object schema validation
const result = validationUtils.object.matchesSchema(config, {
  baseUrl: { type: 'string', required: true },
  timeout: { type: 'number', min: 0 },
  retries: { type: 'number', min: 0, max: 10 }
});

// Array validation
const hasMinLength = validationUtils.array.minLength([1, 2, 3], 2);
const noDuplicates = validationUtils.array.noDuplicates([1, 2, 3, 4]);

// Validate or throw
try {
  validationUtils.assertValid(
    password,
    [
      (pwd) => validationUtils.string.hasMinLength(pwd, 8),
      (pwd) => /[A-Z]/.test(pwd) // Has uppercase letter
    ],
    'Invalid password'
  );
} catch (error) {
  // Handle validation error
}
```

## Integration

### Package Installation

```bash
npm install @bluefly/llmcli
```

### ES Module Import

```typescript
// Import with .js extension for ESM compatibility
import { logger } from '@bluefly/llmcli/shared/logger/index.js';
import { createError } from '@bluefly/llmcli/shared/errors/index.js';
import { HttpClient } from '@bluefly/llmcli/shared/http/index.js';
```

### CommonJS Import

```javascript
// CommonJS fallback import
const { logger } = require('@bluefly/llmcli/shared/logger/index.js');
const { createError } = require('@bluefly/llmcli/shared/errors/index.js');
const { HttpClient } = require('@bluefly/llmcli/shared/http/index.js');
```
